//----------------------------------------------------------------------------

//LED display support module, header file

//----------------------------------------------------------------------------

#ifndef DISPLAY_HPP
#define DISPLAY_HPP

//----------------------------------------------------------------------------
//----------------------------- TSreg class: ---------------------------------
//----------------------------------------------------------------------------

class TSreg
{
private:
  Pin_SCLK_t Pin_SCLK;
  Pin_SDATA_t Pin_SDATA;
  Pin_SS_t Pin_SS;
  Pin_LOAD_t Pin_LOAD;
  Pin_OE_t Pin_OE;
  void SpiWr(uint8_t d);
#ifdef DISP7
  static uint8_t const REGS = 7; //number of registers - 7
#else
  static uint8_t const REGS = 6; //number of registers - 6
#endif
  uint8_t Pos;
public:
  TSreg(void) {};
  void Init(bool on);            //SPI init
  void Load(uint8_t *Value);     //register loading
};

//----------------------------------------------------------------------------
//---------------------------- TDisplay class: -------------------------------
//----------------------------------------------------------------------------


class TDisplay
{
private:
  enum { DIG1, DIG2, DIG3, DIG4, DIG5, DIG6, DIG7, DIGS };
  static uint8_t const POINT = 0x80;   //decimal point code
  static uint8_t const HOME = 0x7F;    //return code to start of string
  static uint8_t const BLANK = 10;     //space code
  static uint8_t const MINUS = 11;     //minus sign code
  static uint8_t const ALT6 = 12;      //alternative digit 6 code
  static uint8_t const ALT7 = 13;      //alternative digit 7 code
  static uint8_t const ALT9 = 14;      //alternative digit 9 code
  uint8_t Digs[DIGS];                  //character buffer
  uint8_t Segs[DIGS];                  //segment buffer
  uint8_t Darks[DIGS];                 //segment buffer for blanking mode
  uint8_t Seven(uint8_t d);            //conversion to 7-segment code
  void Digit(uint8_t d);               //outputting a number to the indicator
  void Digits(uint8_t d);              //filling the indicator with one digit
  void String(uint8_t __flash *s);     //string output from Flash
  void Bcd(uint32_t v, uint8_t *buff); //number conversion to BCD format
  uint8_t Pos;                         //current position
  TSoftTimer *BlinkTimer;              //display blink timer
  static uint16_t const BLINK_TM = 500; //blink period, ms
  bool BlinkEn;                        //blink enable flag
  bool BlinkOn;                        //blink active phase flag
  TSreg Sreg;                          //shift register object
  uint8_t Fnt679;                      //font code for numbers 6, 7, 9
  bool TimeHR;                         //high time resolution
  bool MetrageHR;                      //high metrage resolution
  bool SpeedHR;                        //high speed resolution
  bool SpeedHL;                        //high speed resolution (left shifted)
public:
  TDisplay(void);
  void Execute(void);                  //blink execution
  void Blink(uint8_t p);               //enabling blink of selected digits
  static uint8_t const BLINK = 0x3C;   //blink mask during calibration
  void SetFont(uint8_t fnt);           //font selection for numbers 6, 7, 9
  void SetTimeRes(uint8_t op);         //time display resolution selection
  void SetMetrageRes(uint8_t op);      //metrage display resolution selection
  void SetSpeedRes(uint8_t op);        //speed display resolution selection
  void Clear(void);                    //display cleaning
  void Test(void);                     //display test (switch on all segments)
  void OnOff(bool on);                 //display on/off
  void Text(uint8_t __flash *s);       //text output from Flash
  static bool const LAP = 1;           //LapTime, LapMetr enable code
  void Time(int32_t v, bool lap = 0);  //display time
  void Metrage(int32_t v, bool lap = 0); //display metrage
  void Speed(uint32_t v);              //display speed
  void Diameter(uint32_t v, uint8_t s = 0); //display roller diameter and msg.
  static uint16_t const D_CAL = 1;     //"CAL" message code
  static uint16_t const D_ERR = 2;     //"Err" message code
  void Version(void);                  //display FW version number
};

//----------------------------------------------------------------------------

extern TDisplay *Display;

#define Display_TextF(x) {static __flash uint8_t str[] = {x}; Display->Text(str);}

//----------------------------------------------------------------------------

#endif
