//----------------------------------------------------------------------------

//EEPROM support module

//-----------------------  : ------------------------------

//Built-in EEPROM is used. Data is saved at the moment of power failure.
//At this time, the processor is powered by a special capacitor, which provides
//power during the recording process. This simplifies data integrity control.
//A signature is used to verify the initialization of the EEPROM.

//----------------------------------------------------------------------------

#include "main.hpp"
#include "eeprom.hpp"
#include "count.hpp"

//----------------------------------------------------------------------------
//----------------------------- TEEPROM class: -------------------------------
//----------------------------------------------------------------------------

__no_init __eeprom uint8_t EEData[EE_PARAMS];

//----------------------------- Constructor: ---------------------------------

TEeprom::TEeprom(void)
{
  fValid = 1;
  fValid = (Rd8(EE_SIGN) == EE_SIGNATURE);
}

//----------------------------------------------------------------------------
//-------------------------- Interface methods: ------------------------------
//----------------------------------------------------------------------------

//------------------------ Reading an 8-bit value: ---------------------------

uint8_t TEeprom::Rd8(uint8_t addr, uint8_t def)
{
  if(fValid)
    return(EEData[addr]);
  return(def);
}

//-------------- Reading an 8-bit value with a maximum check: ----------------

uint8_t TEeprom::Rd8Chk(uint8_t addr, uint8_t max, uint8_t def)
{
  uint8_t d = Rd8(addr, def);
  if(d > max) d = def;
  return(d);
}

//------------------------ Writing an 8-bit value: ---------------------------

void TEeprom::Wr8(uint8_t addr, uint8_t val)
{
  if(EEData[addr] != val)
  {
    EEData[addr] = val;
    __watchdog_reset();
  }
}

//------------------------ Reading a 32-bit value: ---------------------------

uint32_t TEeprom::Rd32(uint8_t addr, uint32_t def)
{
  uint32_t val;
  uint8_t *bytes = (uint8_t*)(&val);
  bytes[0] = Rd8(addr++, BYTE1(def));
  bytes[1] = Rd8(addr++, BYTE2(def));
  bytes[2] = Rd8(addr++, BYTE3(def));
  bytes[3] = Rd8(addr,   BYTE4(def));
  return(val);
}

//------------------------ Writing a 32-bit value: ---------------------------

void TEeprom::Wr32(uint8_t addr, uint32_t val)
{
  uint8_t *bytes = (uint8_t*)(&val);
  Wr8(addr++, bytes[0]);
  Wr8(addr++, bytes[1]);
  Wr8(addr++, bytes[2]);
  Wr8(addr,   bytes[3]);
}

//--------------------------- Writing signature: -----------------------------

void TEeprom::Validate(void)
{
  Wr8(EE_SIGN, EE_SIGNATURE);
  fValid = (Rd8(EE_SIGN) == EE_SIGNATURE);
}

//------------------------ Reset the valid flag: -----------------------------

void TEeprom::Invalidate(void)
{
  fValid = 0;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
