//----------------------------------------------------------------------------

//Encoder support module

//-----------------------  : ------------------------------

//The encoder is connected to the ports Pin_F1 (phase 1) and Pin_F2 (phase 2).
//The Execute() method is called in the main loop.
//The encoder processing result is stored in the Message variable.

//----------------------------------------------------------------------------

#include "main.hpp"
#include "encoder.hpp"
#include "setup.hpp"

//----------------------------- Constructor: ---------------------------------

TEncoder::TEncoder(void)
{
  Pin_EF2.DirIn(PIN_PULL);
  Pin_EF1.DirIn(PIN_PULL);
  Message = KEY_NO;
  EncPrev = STATE_0;
  EncPrevPrev = STATE_0;
}

//---------------------- Setting the encoder type: ---------------------------

void TEncoder::SetType(uint8_t t)
{
  EncType = t;
}

//------------------------- Encoder processing: ------------------------------

void TEncoder::Execute(void)
{
  uint8_t EncCur = 0;
  if(!Pin_EF1) EncCur  = STATE_A; //read encoder phase 1
  if(!Pin_EF2) EncCur |= STATE_B; //read encoder phase 2
  if(EncCur != EncPrev)           //if the state has changed, processing
  {
    switch(EncType)
    {
    //encoder type 1
    case OP_ENC1:
      if(EncPrev == STATE_AB &&
         EncCur != EncPrevPrev )
      {
        if(EncCur == STATE_B)
          Message = KEY_UP;
        else
          Message = KEY_DN;
      }
      break;
    //encoder type 2
    case OP_ENC2:
      if(EncPrev == STATE_AB &&
         EncCur != EncPrevPrev )
      {
        if(EncCur == STATE_B)
          Message = KEY_DN;
        else
          Message = KEY_UP;
      }
      break;
    //encoder type 3
    case OP_ENC3:
      if(EncCur == STATE_AB && EncPrevPrev == STATE_0)
      {
        if(EncPrev == STATE_A) Message = KEY_UP;
        if(EncPrev == STATE_B) Message = KEY_DN;
      }
      if(EncCur == STATE_0 && EncPrevPrev == STATE_AB)
      {
        if(EncPrev == STATE_B) Message = KEY_UP;
        if(EncPrev == STATE_A) Message = KEY_DN;
      }
      break;
    //encoder type 4
    case OP_ENC4:
      if(EncCur == STATE_A && EncPrevPrev == STATE_B)
      {
        if(EncPrev == STATE_0) Message = KEY_UP;
        if(EncPrev == STATE_AB) Message = KEY_DN;
      }
      if(EncCur == STATE_B && EncPrevPrev == STATE_A)
      {
        if(EncPrev == STATE_AB) Message = KEY_UP;
        if(EncPrev == STATE_0) Message = KEY_DN;
      }
      break;
    }
    EncPrevPrev = EncPrev; //save pre-previous state
    EncPrev = EncCur;      //save previous state
  }
}

//----------------------------------------------------------------------------
