//----------------------------------------------------------------------------

//Keyboard support module

//-------------------------- Resources used: ---------------------------------

//The button is connected to the Pin_KEY port. When the button is pressed,
//the LOW level appears. The bounce of the button is suppressed by software,
//the time constant is 50 ms. The button generates several events: pressing,
//releasing, holding for 1 second, long holding for 4 seconds.

//----------------------------------------------------------------------------

#include "main.hpp"
#include "keyboard.hpp"

//----------------------------------------------------------------------------
//--------------------------- TKeyboard class: -------------------------------
//----------------------------------------------------------------------------

//----------------------------- Constructor: ---------------------------------

TKeyboard::TKeyboard(void)
{
  Pin_KEY.DirIn(PIN_PULL);
  Message = KEY_NO;
  LastMsg = KEY_NO;
  Prev_Key = 0;
  KeyTimer = new TSoftTimer();
  KeyTimer->Oneshot = 1;
}

//------------------------ Direct read button state: -------------------------

bool TKeyboard::Key(void)
{
  return(!Pin_KEY);
}

//--------------------------- Button handling: -------------------------------

void TKeyboard::Execute(void)
{
  if(TSysTimer::Tick) //every system tick (often not necessary)
  {
    bool Key = !Pin_KEY;               //poll button
    if(Key != Prev_Key)                //state is different
    {
      KeyTimer->Start(DEBOUNCE_TM);    //start debounce timer
      Prev_Key = Key;                  //save new code
    }
    else                               //condition matches
    {
      if(KeyTimer->Over())              //if the bounce is over
      {
        //button press:
        if(Key)
        {
          if(LastMsg == KEY_NO)
          {
            Message = KEY_PRS;         //button press
            LastMsg = KEY_PRS;
            KeyTimer->Start(HOLD_DELAY);
          }
          else if(LastMsg == KEY_PRS)
          {
            Message = KEY_HOLD;        //button hold
            LastMsg = KEY_HOLD;
            KeyTimer->Start(LONG_DELAY);
          }
          else if(LastMsg == KEY_HOLD)
          {
            Message = KEY_LONG;        //long button hold
            LastMsg = KEY_LONG;
          }
        }
        //button release:
        else
        {
          if(LastMsg == KEY_PRS)       //only for short press
          {
            Message = KEY_REL;         //button release
          }
          LastMsg = KEY_NO;
        }
      }
    }
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
