//----------------------------------------------------------------------------

//Main module, header file

//----------------------------------------------------------------------------

#ifndef MAIN_HPP
#define MAIN_HPP

//----------------------------------------------------------------------------

#include <iom168.h>
#include <intrinsics.h>
#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include <stdbool.h>
#include "gpio.hpp"
#include "systimer.hpp"

//-------------- Firmware version and compilation options: -------------------

#define VERSION 204 //firmware version

#define REVOX_A700 //base option
//#define REVOX_B77 224 //Revox B77 option, roller 12..25 mm, 5 blades
//#define TANDBERG 220 //Tandberg TD-20A option, roller 9..16 mm, 2 blades

//#define EN_9_19_38 223 //special option, 9-19-38 cm/s, Count Eable signal

#define COMMONANODE //common anode LEDs are used
#define ENCODER     //encoder control is used

//-------------------------- Processor Pin Usage: ----------------------------

typedef TGpio<PORT_B, PIN_0> Pin_F2_t;    //PB0 (ICP) - F2 OPTO SENSOR [count]
typedef TGpio<PORT_B, PIN_1> Pin_F1_t;    //PB1 - F1 OPTO SENSOR [count]
typedef TGpio<PORT_B, PIN_2> Pin_SS_t;    //PB2 (SS) - SS [display]
typedef TGpio<PORT_B, PIN_3> Pin_SDATA_t; //PB3 (MOSI) - SDATA [display]
typedef TGpio<PORT_B, PIN_4> Pin_PB4_t;   //PB4 (MISO) - NOT USED [control]
typedef TGpio<PORT_B, PIN_5> Pin_SCLK_t;  //PB5 (SCK) - SCLK [display]

typedef TGpio<PORT_C, PIN_0> Pin_LOAD_t;  //PC0 - LOAD [display]
typedef TGpio<PORT_C, PIN_1> Pin_OE_t;    //PC1 - OE [display]
typedef TGpio<PORT_C, PIN_2> Pin_JMP3_t;  //PC2 - JMP3 (19/38) [count]
typedef TGpio<PORT_C, PIN_3> Pin_EF2_t;   //PC3 - JMP2/ENC. F2 [encoder]
typedef TGpio<PORT_C, PIN_4> Pin_EF1_t;   //PC4 - JMP1/ENC. F1 [encoder]
typedef TGpio<PORT_C, PIN_5> Pin_PC5_t;   //PC5 - NOT USED [control]

//typedef TGpio<PORT_D, PIN_0> Pin_RXD_t;   //PD0 (RXD) - RXD [control]
typedef TGpio<PORT_D, PIN_0> Pin_EN_t;   //PD0 - EN [count]
//typedef TGpio<PORT_D, PIN_1> Pin_TXD_t;   //PD1 (TXD) - TXD [control]
typedef TGpio<PORT_D, PIN_1> Pin_19_38_t;   //PD1 - 19/38 [count]
//typedef TGpio<PORT_D, PIN_2> Pin_DIR_t;   //PD2 - DIR [control]
typedef TGpio<PORT_D, PIN_2> Pin_MOVE_t;  //PD2 - MOVE [count]
typedef TGpio<PORT_D, PIN_3> Pin_9_19_t;  //PD3 - 9/19 [count]
typedef TGpio<PORT_D, PIN_4> Pin_KEY_t;   //PD4 - RESET KEY/ENC. KEY [keyboard]
typedef TGpio<PORT_D, PIN_5> Pin_PD5_t;   //PD5 - NOT USED [control]
typedef TGpio<PORT_D, PIN_6> Pin_PD6_t;   //PD6 - NOT USED [control]
typedef TGpio<PORT_D, PIN_7> Pin_VCC_t;   //PD7 (AIN1) - VCC MONITOR [control]

//----------------------- Using Processor Peripherals: -----------------------

//Timer0: system timer [systimer]
//Timer1: tape speed meter [count]
//Timer2: NOT USED
//SPI: display link [display]
//USART: NOT USED
//TWI: NOT USED
//ADC: NOT USED
//Comparator : power monitor [control]

//---------------------------- Using interrupts: -----------------------------

//TIMER0_COMPA: System tick [systimer]
//PCINT0: Opto sensor [count]
//TIMER1_CAPT: Opto sensor [count]

//-------------------------------- Macros: -----------------------------------

#define LO(x)    ((uint8_t)((x) & 0xFF))
#define HI(x)    ((uint8_t)(((x) >> 8) & 0xFF))
#define BYTE1(x) (LO(x))
#define BYTE2(x) (HI(x))
#define BYTE3(x) ((uint8_t)(((uint32_t)(x) >> 16) & 0xFF))
#define BYTE4(x) ((uint8_t)(((uint32_t)(x) >> 24) & 0xFF))

#define WORD(b1,b0)        (((uint16_t)(b1) << 8) | (b0))
#define DWORD(b3,b2,b1,b0) (((uint32_t)WORD(b3,b2) << 16) | WORD(b1,b0))

#define ABS(x) ((x < 0)? -x : x)

#ifndef NULL
  #define NULL _NULL
#endif

//----------------------------------------------------------------------------

#endif
