//----------------------------------------------------------------------------

//Menu implementation module, header file

//----------------------------------------------------------------------------

#ifndef MENU_HPP
#define MENU_HPP

//----------------------------------------------------------------------------

#include "keyboard.hpp"
#include "display.hpp"

//------------------------------- Constants: ---------------------------------

enum Menu_t
{
  //menu indexes:
  MNU_TIME,
  MNU_METRAGE,
  MNU_LAPTIME,
  MNU_LAPMETRAGE,
  MNU_SPEED,
  MNU_DIAMETER,
  MNU_SETUP,
  //number of menus:
  MENUS,
  //special codes:
  MNU_BACK
};

//----------------------------------------------------------------------------
//----------------------- TMenuItem abstract class: --------------------------
//----------------------------------------------------------------------------

class TMenuItem
{
private:
protected:
public:
  TMenuItem(void) {};
  uint8_t MnuIndex;
  virtual void Brief(void) = 0;
  virtual void Show(void) = 0;
  virtual void Update(void) = 0;
  virtual void OnKeyboard(KeyMsg_t &msg) = 0;
};

//----------------------------------------------------------------------------
//--------------------------- TMenuTime class: -------------------------------
//----------------------------------------------------------------------------

class TMenuTime : public TMenuItem
{
private:
public:
  TMenuTime(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//-------------------------- TMenuMetrage class: -----------------------------
//----------------------------------------------------------------------------

class TMenuMetrage : public TMenuItem
{
private:
public:
  TMenuMetrage(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//-------------------------- TMenuLapTime class: -----------------------------
//----------------------------------------------------------------------------

class TMenuLapTime : public TMenuItem
{
private:
public:
  TMenuLapTime(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//------------------------ TMenuLapMetrage class: ----------------------------
//----------------------------------------------------------------------------

class TMenuLapMetrage : public TMenuItem
{
private:
public:
  TMenuLapMetrage(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//--------------------------- TMenuSpeed class: ------------------------------
//----------------------------------------------------------------------------

class TMenuSpeed : public TMenuItem
{
private:
public:
  TMenuSpeed(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//-------------------------- TMenuDiameter class: ----------------------------
//----------------------------------------------------------------------------

class TMenuDiameter : public TMenuItem
{
private:
public:
  TMenuDiameter(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//--------------------------- TMenuSetup class: ------------------------------
//----------------------------------------------------------------------------

class TMenuSetup : public TMenuItem
{
private:
  uint8_t SubMenu;
public:
  TMenuSetup(void) {};
  virtual void Brief(void);
  virtual void Show(void);
  virtual void Update(void);
  virtual void OnKeyboard(KeyMsg_t &msg);
};

//----------------------------------------------------------------------------
//-------------------------- TList template class: ---------------------------
//----------------------------------------------------------------------------

template<class T>
class TList
{
private:
  char ItemsMax;
public:
  TList(char max);
  T** Items;
  char ItemsCount;
  void AddItem(T *t);
};

//------------------------- Methods implementation: --------------------------

template<class T>
TList<T>::TList(char max)
{
  ItemsMax = max;
  Items = new T*[ItemsMax];
  ItemsCount = 0;
}

template<class T>
void TList<T>::AddItem(T *t)
{
  if(ItemsCount < ItemsMax)
    Items[ItemsCount++] = t;
}

//----------------------------------------------------------------------------
//-------------------------- TMenuItems class: -------------------------------
//----------------------------------------------------------------------------

class TMenuItems : public TList<TMenuItem>
{
public:
  TMenuItems(char max);
  void SelectMenu(uint8_t mnu);
  TMenuItem *SelectedMenu;
};

//----------------------------------------------------------------------------

#endif
