//----------------------------------------------------------------------------

//Settings module

//----------------------------------------------------------------------------

#include "main.hpp"
#include "setup.hpp"
#include "display.hpp"
#include "count.hpp"
#include "eeprom.hpp"
#include "menu.hpp"
#include "encoder.hpp"

//----------------------------------------------------------------------------
//----------------------------- TSetup class: --------------------------------
//----------------------------------------------------------------------------

//----------------------------- Constructor: ---------------------------------

TSetup::TSetup(void)
{
  EERead();
  Apply();
}

//----------------------------- Read EEPROM: ---------------------------------

void TSetup::EERead(void)
{
  Op_Menus  = Eeprom->Rd8Chk(EE_OP_MENUS,  OP_MENUS_MAX,  OP_TLTLS);
#ifdef ENCODER
  Op_RtcRes = Eeprom->Rd8Chk(EE_OP_RTCRES, OP_RTCRES_MAX, OP_RTCHI);
  Op_LenRes = Eeprom->Rd8Chk(EE_OP_LENRES, OP_LENRES_MAX, OP_LENHI);
  Op_SpdRes = Eeprom->Rd8Chk(EE_OP_SPDRES, OP_SPDRES_MAX, OP_SPDHI);
#else
  Op_RtcRes = Eeprom->Rd8Chk(EE_OP_RTCRES, OP_RTCRES_MAX, OP_RTCLO);
  Op_LenRes = Eeprom->Rd8Chk(EE_OP_LENRES, OP_LENRES_MAX, OP_LENLO);
  Op_SpdRes = Eeprom->Rd8Chk(EE_OP_SPDRES, OP_SPDRES_MAX, OP_SPDHL);
#endif
  Op_Font   = Eeprom->Rd8Chk(EE_OP_FONT,   OP_FONT_MAX,   OP_A69);
  Op_SpdSel = Eeprom->Rd8Chk(EE_OP_SPSEL,  OP_SPSEL_MAX,  OP_H19L9);
  Op_Loop   = Eeprom->Rd8Chk(EE_OP_LOOP,   OP_LOOP_MAX,   OP_LINE);
  //Op_Enc    = Eeprom->Rd8Chk(EE_OP_ENC,    OP_ENC_MAX,    OP_ENC2);
  Op_Enc    = Eeprom->Rd8Chk(EE_OP_ENC,    OP_ENC_MAX,    OP_ENC1);
}

//--------------------------- Save to EEPROM: --------------------------------

void TSetup::EESave(void)
{
  Eeprom->Wr8(EE_OP_MENUS,  Op_Menus);
  Eeprom->Wr8(EE_OP_RTCRES, Op_RtcRes);
  Eeprom->Wr8(EE_OP_LENRES, Op_LenRes);
  Eeprom->Wr8(EE_OP_SPDRES, Op_SpdRes);
  Eeprom->Wr8(EE_OP_FONT,   Op_Font);
  Eeprom->Wr8(EE_OP_SPSEL,  Op_SpdSel);
  Eeprom->Wr8(EE_OP_LOOP,   Op_Loop);
  Eeprom->Wr8(EE_OP_ENC,    Op_Enc);
}

//--------------------- Loading options into modules: ------------------------

void TSetup::Apply(void)
{
  Display->SetTimeRes(Op_RtcRes);
  Display->SetMetrageRes(Op_LenRes);
  Display->SetSpeedRes(Op_SpdRes);
  Display->SetFont(Op_Font);
  Count->SetSpdSel(Op_SpdSel);
  Encoder->SetType(Op_Enc);
}

//---------------------- Indication of Menus values: -------------------------

void TSetup::ShowMenus()
{
  switch(Op_Menus)
  {
  case OP_TLTLS: Display_TextF(" tLtLS "); break;
  case OP_TLTS:  Display_TextF(" tLtS  "); break;
  case OP_TLS:   Display_TextF(" tLS   "); break;
  case OP_TTS:   Display_TextF(" ttS   "); break;
  case OP_TS:    Display_TextF(" tS    "); break;
  case OP_T:     Display_TextF(" t     "); break;
  }
}

//---------------------- Indication of RtcRes values: ------------------------

void TSetup::ShowRtcRes()
{
  switch(Op_RtcRes)
  {
  case OP_RTCLO: Display_TextF(" rtc   "); break;
  case OP_RTCHI: Display_TextF(" rtcHI "); break;
  }
}

//---------------------- Indication of LenRes values: ------------------------

void TSetup::ShowLenRes()
{
  switch(Op_LenRes)
  {
  case OP_LENLO: Display_TextF(" LEn   "); break;
  case OP_LENHI: Display_TextF(" LEnHI "); break;
  }
}

//---------------------- Indication of SpdRes values: ------------------------

void TSetup::ShowSpdRes()
{
  switch(Op_SpdRes)
  {
  case OP_SPDLO: Display_TextF(" SPd   "); break;
  case OP_SPDHI: Display_TextF(" SPdHI "); break;
  case OP_SPDHL: Display_TextF(" SPdHL "); break;
  }
}

//----------------------- Indication of Font values: -------------------------

void TSetup::ShowFont()
{
  Display->SetFont(Op_Font);
  Display_TextF("  769  ");
}

//---------------------- Indication of SpSel values: -------------------------

void TSetup::ShowSpSel()
{
  switch(Op_SpdSel)
  {
  case OP_H19L9:  Display_TextF(" H19L9 "); break;
  case OP_H9L19:  Display_TextF(" H9L19 "); break;
  case OP_H38L19: Display_TextF("H38L19 "); break;
  case OP_H19L38: Display_TextF("H19L38 "); break;
  }
}

//---------------------- Indication of Loop values: --------------------------

void TSetup::ShowLoop()
{
  switch(Op_Loop)
  {
  case OP_LOOP: Display_TextF(" CSEL  "); break;
  case OP_LINE: Display_TextF(" LSEL  "); break;
  }
}

//---------------------- Indication of Enc values: ---------------------------

void TSetup::ShowEnc()
{
  Encoder->SetType(Op_Enc);
  switch(Op_Enc)
  {
  case OP_ENC1: Display_TextF(" Enc1  "); break;
  case OP_ENC2: Display_TextF(" Enc2  "); break;
  case OP_ENC3: Display_TextF(" Enc3  "); break;
  case OP_ENC4: Display_TextF(" Enc4  "); break;
  }
}

//----------------------------------------------------------------------------
//-------------------------- Interface methods: ------------------------------
//----------------------------------------------------------------------------

//------------------------ Setup menu item display: --------------------------

void TSetup::Show(uint8_t m)
{
  switch(m)
  {
  case SM_ESC:    Display_TextF("  ESC  "); break;
  case SM_MENUS:  ShowMenus();  break;
  case SM_RTCRES: ShowRtcRes(); break;
  case SM_LENRES: ShowLenRes(); break;
  case SM_SPDRES: ShowSpdRes(); break;
  case SM_FONT:   ShowFont();   break;
  case SM_SPDSEL: ShowSpSel();  break;
  case SM_LOOP:   ShowLoop();   break;
  case SM_ENC:    ShowEnc();    break;
  case SM_SET:    Display_TextF("  SEt  "); break;
  case SM_DEF:    Display_TextF("  dEF  "); break;
  }
}

//----------------------------- Editing values: ------------------------------

void TSetup::Edit(uint8_t m)
{
  switch(m)
  {
  case SM_MENUS:  if(++Op_Menus  == OP_MENUS_MAX)  Op_Menus  = 0; break;
  case SM_RTCRES: if(++Op_RtcRes == OP_RTCRES_MAX) Op_RtcRes = 0; break;
  case SM_LENRES: if(++Op_LenRes == OP_LENRES_MAX) Op_LenRes = 0; break;
  case SM_SPDRES: if(++Op_SpdRes == OP_SPDRES_MAX) Op_SpdRes = 0; break;
  case SM_FONT:   if(++Op_Font   == OP_FONT_MAX)   Op_Font   = 0; break;
  case SM_SPDSEL: if(++Op_SpdSel == OP_SPSEL_MAX)  Op_SpdSel = 0; break;
  case SM_LOOP:   if(++Op_Loop   == OP_LOOP_MAX)   Op_Loop   = 0; break;
  case SM_ENC:    if(++Op_Enc    == OP_ENC_MAX)    Op_Enc    = 0; break;
  }
}

//---------------- Matrix of transitions to the next menu: -------------------

uint8_t TSetup::NextMenu(uint8_t mnu)
{
  uint8_t next = MNU_TIME;
  switch(mnu)
  {
  case MNU_TIME:
    if(Op_Menus < OP_TTS) next = MNU_METRAGE;
      else if(Op_Menus == OP_TTS) next = MNU_LAPTIME;
        else if(Op_Menus == OP_TS) next = MNU_SPEED;
    break;
  case MNU_METRAGE:
    if(Op_Menus < OP_TLS) next = MNU_LAPTIME;
      else if(Op_Menus == OP_TLS) next = MNU_SPEED;
    break;
  case MNU_LAPTIME:
    if(Op_Menus == OP_TLTLS) next = MNU_LAPMETRAGE;
      else if((Op_Menus == OP_TLTS) || (Op_Menus == OP_TTS)) next = MNU_SPEED;
    break;
  case MNU_LAPMETRAGE:
    if(Op_Menus == OP_TLTLS) next = MNU_SPEED;
    break;
  case MNU_SPEED:
#ifdef ENCODER
    if((Op_Menus < OP_T) && (Op_Loop == OP_LINE)) next = MNU_SPEED;
#endif
    break;
  }
  return(next);
}

//-------------- Matrix of transitions to the previous menu: -----------------

uint8_t TSetup::PrevMenu(uint8_t mnu)
{
  uint8_t prev = MNU_TIME;
  switch(mnu)
  {
  case MNU_TIME:
    if((Op_Menus < OP_T) && (Op_Loop == OP_LOOP)) prev = MNU_SPEED;
    break;
  case MNU_LAPTIME:
    if(Op_Menus < OP_TLS) prev = MNU_METRAGE;
    break;
  case MNU_LAPMETRAGE:
    if(Op_Menus == OP_TLTLS) prev = MNU_LAPTIME;
    break;
  case MNU_SPEED:
    if(Op_Menus == OP_TLTLS) prev = MNU_LAPMETRAGE;
      else if((Op_Menus == OP_TLTS) || (Op_Menus == OP_TTS)) prev = MNU_LAPTIME;
        else if(Op_Menus == OP_TLS) prev = MNU_METRAGE;
    break;
  }
  return(prev);
}

//-------------------------- Rejection of changes: ---------------------------

void TSetup::Esc(void)
{
  EERead();
  Display->SetFont(Op_Font);
  Encoder->SetType(Op_Enc);
}

//--------------------------- Loading new values: ----------------------------

void TSetup::Set(void)
{
  EESave();
  Apply();
}

//--------------------------- Loading defaults: ------------------------------

void TSetup::Def(void)
{
  Eeprom->Invalidate();
  EERead();
  Eeprom->Validate();
  Set();
}

//----------------------------------------------------------------------------
