//----------------------------------------------------------------------------

//Settings module, header file

//----------------------------------------------------------------------------

#ifndef SETUP_HPP
#define SETUP_HPP

//------------------------------- Constants: ---------------------------------

//Setup menu items:
enum
{
  SM_ESC,    //exit without saving
  SM_MENUS,  //selection of a set of displayed values
  SM_RTCRES, //enable high time resolution
  SM_LENRES, //enable high length resolution
  SM_SPDRES, //enable high speed resolution
  SM_FONT,   //font selection
  SM_SPDSEL, //selection of speed switching logic
  SM_LOOP,   //enabling display looping
  SM_ENC,    //encoder type selection
  SM_SET,    //exit with saving
  SM_DEF,    //loading default values

  SM_MIN = SM_ESC,
  SM_MAX = SM_DEF
};

//Options codes:
enum { OP_TLTLS, OP_TLTS, OP_TLS , OP_TTS, OP_TS, OP_T, OP_MENUS_MAX };
enum { OP_RTCLO, OP_RTCHI, OP_RTCRES_MAX };
enum { OP_LENLO, OP_LENHI, OP_LENRES_MAX };
enum { OP_SPDLO, OP_SPDHI, OP_SPDHL, OP_SPDRES_MAX };
enum { OP_769, OP_A69, OP_7AA, OP_AAA, OP_FONT_MAX };
enum { OP_H19L9, OP_H9L19, OP_H38L19, OP_H19L38, OP_SPSEL_MAX };
enum { OP_LOOP, OP_LINE, OP_LOOP_MAX };
enum { OP_ENC1, OP_ENC2, OP_ENC3, OP_ENC4, OP_ENC_MAX };

//----------------------------------------------------------------------------
//----------------------------- TSetup class: --------------------------------
//----------------------------------------------------------------------------

class TSetup
{
private:
  uint8_t Op_Menus;
  void ShowMenus();
  uint8_t Op_RtcRes;
  void ShowRtcRes();
  uint8_t Op_LenRes;
  void ShowLenRes();
  uint8_t Op_SpdRes;
  void ShowSpdRes();
  uint8_t Op_Font;
  void ShowFont();
  uint8_t Op_SpdSel;
  void ShowSpSel();
  uint8_t Op_Loop;
  void ShowLoop();
  uint8_t Op_Enc;
  void ShowEnc();
  void EERead(void);
  void EESave(void);
  void Apply(void);
public:
  TSetup(void);
  void Show(uint8_t m);
  void Edit(uint8_t m);
  uint8_t NextMenu(uint8_t mnu);
  uint8_t PrevMenu(uint8_t mnu);
  void Esc(void);
  void Def(void);
  void Set(void);
};

//----------------------------------------------------------------------------

extern TSetup *Setup;

//----------------------------------------------------------------------------

#endif

